searchData={"items":[{"type":"module","title":"Hammer","doc":"Documentation for Hammer module.\n\nThis is the main API for the Hammer rate-limiter. This module assumes a\nbackend pool has been started, most likely by the Hammer application.","ref":"Hammer.html"},{"type":"function","title":"Hammer.check_rate/3","doc":"Check if the action you wish to perform is within the bounds of the rate-limit.\n\nArgs:\n- `id`: String name of the bucket. Usually the bucket name is comprised of\nsome fixed prefix, with some dynamic string appended, such as an IP address or\nuser id.\n- `scale_ms`: Integer indicating size of bucket in milliseconds\n- `limit`: Integer maximum count of actions within the bucket\n\nReturns either `{:allow,  count}`, `{:deny,   limit}` or `{:error,  reason}`\n\nExample:\n\n    user_id = 42076\n    case check_rate(\"file_upload:#{user_id}\", 60_000, 5) do\n      {:allow, _count} ->\n        # do the file upload\n      {:deny, _limit} ->\n        # render an error page or something\n    end","ref":"Hammer.html#check_rate/3"},{"type":"function","title":"Hammer.check_rate/4","doc":"Same as `check_rate/3`, but allows specifying a backend.","ref":"Hammer.html#check_rate/4"},{"type":"function","title":"Hammer.check_rate_inc/4","doc":"Same as check_rate/3, but allows the increment number to be specified.\nThis is useful for limiting apis which have some idea of 'cost', where the cost\nof each hit can be specified.","ref":"Hammer.html#check_rate_inc/4"},{"type":"function","title":"Hammer.check_rate_inc/5","doc":"Same as check_rate_inc/4, but allows specifying a backend.","ref":"Hammer.html#check_rate_inc/5"},{"type":"function","title":"Hammer.delete_buckets/1","doc":"Delete all buckets belonging to the provided id, including the current one.\nEffectively resets the rate-limit for the id.\n\nArguments:\n\n- `id`: String name of the bucket\n\nReturns either `{:ok, count}` where count is the number of buckets deleted,\nor `{:error, reason}`.\n\nExample:\n\n    user_id = 2406\n    {:ok, _count} = delete_buckets(\"file_uploads:#{user_id}\")","ref":"Hammer.html#delete_buckets/1"},{"type":"function","title":"Hammer.delete_buckets/2","doc":"Same as delete_buckets/1, but allows specifying a backend","ref":"Hammer.html#delete_buckets/2"},{"type":"function","title":"Hammer.inspect_bucket/3","doc":"Inspect bucket to get count, count_remaining, ms_to_next_bucket, created_at,\nupdated_at. This function is free of side-effects and should be called with\nthe same arguments you would use for `check_rate` if you intended to increment\nand check the bucket counter.\n\nArguments:\n\n- `id`: String name of the bucket. Usually the bucket name is comprised of\n  some fixed prefix,with some dynamic string appended, such as an IP address\n  or user id.\n- `scale_ms`: Integer indicating size of bucket in milliseconds\n- `limit`: Integer maximum count of actions within the bucket\n\nReturns either\n`{:ok, {count, count_remaining, ms_to_next_bucket, created_at, updated_at}`,\nor `{:error, reason}`.\n\nExample:\n\n    inspect_bucket(\"file_upload:2042\", 60_000, 5)\n    {:ok, {1, 2499, 29381612, 1450281014468, 1450281014468}}","ref":"Hammer.html#inspect_bucket/3"},{"type":"function","title":"Hammer.inspect_bucket/4","doc":"Same as inspect_bucket/3, but allows specifying a backend","ref":"Hammer.html#inspect_bucket/4"},{"type":"function","title":"Hammer.make_rate_checker/3","doc":"Make a rate-checker function, with the given `id` prefix, scale_ms and limit.\n\nArguments:\n\n- `id_prefix`: String prefix to the `id`\n- `scale_ms`: Integer indicating size of bucket in milliseconds\n- `limit`: Integer maximum count of actions within the bucket\n\nReturns a function which accepts an `id` suffix, which will be combined with\nthe `id_prefix`. Calling this returned function is equivalent to:\n`Hammer.check_rate(\"#{id_prefix}#{id}\", scale_ms, limit)`\n\nExample:\n\n    chat_rate_limiter = make_rate_checker(\"send_chat_message:\", 60_000, 20)\n    user_id = 203517\n    case chat_rate_limiter.(user_id) do\n      {:allow, _count} ->\n        # allow chat message\n      {:deny, _limit} ->\n        # deny\n    end","ref":"Hammer.html#make_rate_checker/3"},{"type":"function","title":"Hammer.make_rate_checker/4","doc":"","ref":"Hammer.html#make_rate_checker/4"},{"type":"module","title":"Hammer.Application","doc":"Hammer application, responsible for starting the backend worker pools.\n\nConfigured with the `:hammer` environment key:\n\n- `:backend`, Either a tuple of `{module, config}`, or a keyword-list\n  of separate, named backends. Examples:\n  `{Hammer.Backend.ETS, []}`, `[ets: {Hammer.Backend.ETS, []}, ...]`\n- `:suppress_logs`, if set to `true`, stops all log messages from Hammer\n\n#","ref":"Hammer.Application.html"},{"type":"module","title":"General Backend Options - Hammer.Application","doc":"Different backends take different options, but all will accept the following\noptions, and with the same effect:\n\n- `:expiry_ms` (int): expiry time in milliseconds, after which a bucket will\n  be deleted. The exact mechanism for cleanup will vary by backend. This configuration\n  option is mandatory\n- `:pool_size` (int): size of the backend worker pool (default=2)\n- `:pool_max_overflow` int(): number of extra workers the pool is permitted\n  to spawn when under pressure. The worker pool (managed by the poolboy library)\n  will automatically create and destroy workers up to the max-overflow limit\n  (default=0)\n\nExample of a single backend:\n\n    config :hammer,\n      backend: {Hammer.Backend.ETS, [expiry_ms: 60_000 * 60 * 2]}\n\nExample of config for multiple-backends:\n\n    config :hammer,\n      backend: [\n        ets: {\n          Hammer.Backend.ETS,\n          [\n            ets_table_name: :hammer_backend_ets_buckets,\n            expiry_ms: 60_000 * 60 * 2,\n            cleanup_interval_ms: 60_000 * 2,\n          ]\n        },\n        redis: {\n          Hammer.Backend.Redis,\n          [\n            expiry_ms: 60_000 * 60 * 2,\n            redix_config: [host: \"localhost\", port: 6379],\n            pool_size: 4,\n          ]\n        }\n      ]","ref":"Hammer.Application.html#module-general-backend-options"},{"type":"function","title":"Hammer.Application.start/2","doc":"","ref":"Hammer.Application.html#start/2"},{"type":"behaviour","title":"Hammer.Backend","doc":"The backend Behaviour module.","ref":"Hammer.Backend.html"},{"type":"callback","title":"Hammer.Backend.count_hit/3","doc":"","ref":"Hammer.Backend.html#c:count_hit/3"},{"type":"callback","title":"Hammer.Backend.count_hit/4","doc":"","ref":"Hammer.Backend.html#c:count_hit/4"},{"type":"callback","title":"Hammer.Backend.delete_buckets/2","doc":"","ref":"Hammer.Backend.html#c:delete_buckets/2"},{"type":"callback","title":"Hammer.Backend.get_bucket/2","doc":"","ref":"Hammer.Backend.html#c:get_bucket/2"},{"type":"type","title":"Hammer.Backend.bucket_info/0","doc":"","ref":"Hammer.Backend.html#t:bucket_info/0"},{"type":"type","title":"Hammer.Backend.bucket_key/0","doc":"","ref":"Hammer.Backend.html#t:bucket_key/0"},{"type":"module","title":"Hammer.Backend.ETS","doc":"An ETS backend for Hammer.\n\nThe public API of this module is used by Hammer to store information about\nrate-limit 'buckets'. A bucket is identified by a `key`, which is a tuple\n`{bucket_number, id}`. The essential schema of a bucket is:\n`{key, count, created_at, updated_at}`, although backends are free to\nstore and retrieve this data in whichever way they wish.\n\nUse `start` or `start_link` to start the server:\n\n    {:ok, pid} = Hammer.Backend.ETS.start_link(args)\n\n`args` is a keyword list:\n- `expiry_ms`: (integer) time in ms before a bucket is auto-deleted,\n  should be larger than the expected largest size/duration of a bucket\n- `cleanup_interval_ms`: (integer) time between cleanup runs,\n- `ets_table_type`: (atom) type of ETS table, defaults to `:set`, and can be\n  either `:set` or `:ordered_set`\n\nExample:\n\n    Hammer.Backend.ETS.start_link(\n      expiry_ms: 1000 * 60 * 60,\n      cleanup_interval_ms: 1000 * 60 * 10\n    )","ref":"Hammer.Backend.ETS.html"},{"type":"function","title":"Hammer.Backend.ETS.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Hammer.Backend.ETS.html#child_spec/1"},{"type":"function","title":"Hammer.Backend.ETS.count_hit/3","doc":"Record a hit in the bucket identified by `key`","ref":"Hammer.Backend.ETS.html#count_hit/3"},{"type":"function","title":"Hammer.Backend.ETS.count_hit/4","doc":"Record a hit in the bucket identified by `key`, with a custom increment","ref":"Hammer.Backend.ETS.html#count_hit/4"},{"type":"function","title":"Hammer.Backend.ETS.delete_buckets/2","doc":"Delete all buckets associated with `id`.","ref":"Hammer.Backend.ETS.html#delete_buckets/2"},{"type":"function","title":"Hammer.Backend.ETS.get_bucket/2","doc":"Retrieve information about the bucket identified by `key`","ref":"Hammer.Backend.ETS.html#get_bucket/2"},{"type":"function","title":"Hammer.Backend.ETS.init/1","doc":"","ref":"Hammer.Backend.ETS.html#init/1"},{"type":"function","title":"Hammer.Backend.ETS.start/0","doc":"","ref":"Hammer.Backend.ETS.html#start/0"},{"type":"function","title":"Hammer.Backend.ETS.start/1","doc":"","ref":"Hammer.Backend.ETS.html#start/1"},{"type":"function","title":"Hammer.Backend.ETS.start_link/0","doc":"","ref":"Hammer.Backend.ETS.html#start_link/0"},{"type":"function","title":"Hammer.Backend.ETS.start_link/1","doc":"","ref":"Hammer.Backend.ETS.html#start_link/1"},{"type":"function","title":"Hammer.Backend.ETS.stop/0","doc":"","ref":"Hammer.Backend.ETS.html#stop/0"},{"type":"type","title":"Hammer.Backend.ETS.bucket_info/0","doc":"","ref":"Hammer.Backend.ETS.html#t:bucket_info/0"},{"type":"type","title":"Hammer.Backend.ETS.bucket_key/0","doc":"","ref":"Hammer.Backend.ETS.html#t:bucket_key/0"},{"type":"module","title":"Hammer.Supervisor","doc":"Top-level Supervisor for the Hammer application.\n\nStarts a set of poolboy pools based on provided configuration,\nwhich are latter called to by the `Hammer` module.\nSee the Application module for configuration examples.","ref":"Hammer.Supervisor.html"},{"type":"function","title":"Hammer.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Hammer.Supervisor.html#child_spec/1"},{"type":"function","title":"Hammer.Supervisor.init/1","doc":"","ref":"Hammer.Supervisor.html#init/1"},{"type":"function","title":"Hammer.Supervisor.start_link/2","doc":"","ref":"Hammer.Supervisor.html#start_link/2"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"Unreleased - Changelog","doc":"","ref":"changelog.html#unreleased"},{"type":"extras","title":"6.2.1 - 2024-02-23 - Changelog","doc":"- Fix issue in OTP 26 and Elixir 1.15 by not using to_existing_atom in configuration\n\n#","ref":"changelog.html#6-2-1-2024-02-23"},{"type":"extras","title":"Changed - Changelog","doc":"","ref":"changelog.html#changed"},{"type":"extras","title":"6.2.0 - 2024-01-31 - Changelog","doc":"- Ensure Elixir version is ~> 1.13 https://github.com/ExHammer/hammer/pull/79.","ref":"changelog.html#6-2-0-2024-01-31"},{"type":"extras","title":"6.1.0 - 2022-06-13 - Changelog","doc":"#","ref":"changelog.html#6-1-0-2022-06-13"},{"type":"extras","title":"Changed - Changelog","doc":"- Updgrade dependency packages\n- Merged https://github.com/ExHammer/hammer/pull/41 resulting in ETC without GenServer (and therefore better performance)\n- Merged https://github.com/ExHammer/hammer/pull/46 remove additional whitespace\n- Updated Docs based on https://github.com/ExHammer/hammer/pull/45\n- Adds CREDITS.md","ref":"changelog.html#changed"},{"type":"extras","title":"6.0.0 - 2018-10-12 - Changelog","doc":"#","ref":"changelog.html#6-0-0-2018-10-12"},{"type":"extras","title":"Changed - Changelog","doc":"- Change the `ETS` backend to throw an error if either `expiry_ms` or\n  `cleanup_interval_ms` config values are missing. This should have been fixed\n  ages ago.\n- Default `:pool_max_overflow` changed to `0`. It's a better default, given\n  that some users have seen weird errors when using a higher overflow.\n  In general, capacity should be increased by using a higher `:pool_size` instead\n- Changed how the ETS backend does cleanups of data, should be more performant.","ref":"changelog.html#changed"},{"type":"extras","title":"5.0.0 - 2018-05-18 - Changelog","doc":"#","ref":"changelog.html#5-0-0-2018-05-18"},{"type":"extras","title":"Added - Changelog","doc":"- A new `check_rate_inc` function, which allows the caller to specify the\n  integer with which to increment the bucket by. This is useful for limiting\n  APIs which have some notion of \"cost\" per call.","ref":"changelog.html#added"},{"type":"extras","title":"4.0.0 - 2018-04-23 - Changelog","doc":"#","ref":"changelog.html#4-0-0-2018-04-23"},{"type":"extras","title":"Changed - Changelog","doc":"- Use a worker-pool for the backend (via poolboy),\n  this avoids bottle-necking all traffic through a single hammer\n  process, thus improving throughput for the system overall\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- New configuration options for backends:\n  - `:pool_size`, determines the number of workers in the pool (default 4)\n  - `:pool_max_overflow`, maximum extra workers to be spawned when the\n    system is under pressure (default 4)\n- Multiple instances of the same backend! You can now have two ETS backends,\n  fifteen Redis's, whatever you want","ref":"changelog.html#added"},{"type":"extras","title":"3.0.0 - 2018-02-20 - Changelog","doc":"#","ref":"changelog.html#3-0-0-2018-02-20"},{"type":"extras","title":"Changed - Changelog","doc":"- Require elixir >= 1.6\n- Use a more sane supervision tree structure","ref":"changelog.html#changed"},{"type":"extras","title":"2.1.0 2017-11-25 - Changelog","doc":"#","ref":"changelog.html#2-1-0-2017-11-25"},{"type":"extras","title":"Changed - Changelog","doc":"- Add option to use more than one backend\n- Add option to suppress all logging","ref":"changelog.html#changed"},{"type":"extras","title":"2.0.0 - 2017-09-24 - Changelog","doc":"#","ref":"changelog.html#2-0-0-2017-09-24"},{"type":"extras","title":"Changed - Changelog","doc":"- New, simpler API\n  - No longer need to start backend processes manually\n  - Call `Hammer.check_rate` directly, rather than `use`ing a macro\n- Hammer is now an OTP application, configured via `Mix.Config`","ref":"changelog.html#changed"},{"type":"extras","title":"1.0.0 - 2017-08-22 - Changelog","doc":"#","ref":"changelog.html#1-0-0-2017-08-22"},{"type":"extras","title":"Added - Changelog","doc":"- Formalise backend API in `Hammer.Backend` behaviour","ref":"changelog.html#added"},{"type":"extras","title":"0.2.1 - 2017-08-10 - Changelog","doc":"#","ref":"changelog.html#0-2-1-2017-08-10"},{"type":"extras","title":"Changed - Changelog","doc":"- Minor fixes","ref":"changelog.html#changed"},{"type":"extras","title":"Readme","doc":"# Hammer\n\n[![Build Status](https://github.com/ExHammer/hammer/actions/workflows/ci.yml/badge.svg)](https://github.com/ExHammer/hammer/actions/workflows/ci.yml) [![Hex.pm](https://img.shields.io/hexpm/v/hammer.svg)](https://hex.pm/packages/hammer) [![Documentation](https://img.shields.io/badge/documentation-gray)](https://hexdocs.pm/hammer)\n[![Total Download](https://img.shields.io/hexpm/dt/hammer.svg)](https://hex.pm/packages/hammer)\n[![License](https://img.shields.io/hexpm/l/hammer.svg)](https://github.com/ExHammer/hammer/blob/master/LICENSE.md)\n\nA rate-limiter for Elixir, with pluggable storage backends.","ref":"readme.html"},{"type":"extras","title":"Hammer-Plug - Readme","doc":"We have a helper-library to make adding rate-limiting to your Phoenix\n(or other plug-based) application even easier: [Hammer.Plug](https://github.com/ExHammer/hammer-plug).","ref":"readme.html#hammer-plug"},{"type":"extras","title":"Installation - Readme","doc":"Hammer is [available in Hex](https://hex.pm/packages/hammer), the package can be installed\nby adding `:hammer` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:hammer, \"~> 6.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Documentation - Readme","doc":"On HexDocs: [https://hexdocs.pm/hammer/frontpage.html](https://hexdocs.pm/hammer/frontpage.html)\n\nThe [Tutorial](https://hexdocs.pm/hammer/tutorial.html) is an especially good place to start.","ref":"readme.html#documentation"},{"type":"extras","title":"Usage - Readme","doc":"Example:\n\n```elixir\ndefmodule MyApp.VideoUpload do\n\n  def upload(video_data, user_id) do\n    case Hammer.check_rate(\"upload_video:#{user_id}\", 60_000, 5) do\n      {:allow, _count} ->\n        # upload the video, somehow\n      {:deny, _limit} ->\n        # deny the request\n    end\n  end\n\nend\n```\n\nThe `Hammer` module provides the following functions:\n\n- `check_rate(id, scale_ms, limit)`\n- `check_rate_inc(id, scale_ms, limit, increment)`\n- `inspect_bucket(id, scale_ms, limit)`\n- `delete_buckets(id)`\n\nBackends are configured via `Mix.Config`:\n\n```elixir\nconfig :hammer,\n  backend: {Hammer.Backend.ETS, [expiry_ms: 60_000 * 60 * 4,\n                                 cleanup_interval_ms: 60_000 * 10]}\n```\n\nSee the [Tutorial](https://hexdocs.pm/hammer/tutorial.html) for more.\n\nSee the [Hammer Testbed](https://github.com/ExHammer/hammer-testbed) app for an example of\nusing Hammer in a Phoenix application.","ref":"readme.html#usage"},{"type":"extras","title":"Available Backends - Readme","doc":"- [Hammer.Backend.ETS](https://hexdocs.pm/hammer/Hammer.Backend.ETS.html) (provided with Hammer for testing and dev purposes, not very good for production use)\n- [Hammer.Backend.Redis](https://github.com/ExHammer/hammer-backend-redis)\n- [Hammer.Backend.Mnesia](https://github.com/ExHammer/hammer-backend-mnesia) (beta)","ref":"readme.html#available-backends"},{"type":"extras","title":"Getting Help - Readme","doc":"If you're having trouble, either open an issue on this repo","ref":"readme.html#getting-help"},{"type":"extras","title":"Acknowledgements - Readme","doc":"Hammer was inspired by the [ExRated](https://github.com/grempe/ex_rated) library, by [grempe](https://github.com/grempe).","ref":"readme.html#acknowledgements"},{"type":"extras","title":"License - Readme","doc":"Copyright (c) 2023 June Kelly\n\nThis library is MIT licensed. See the [LICENSE](https://github.com/ExHammer/hammer/blob/master/LICENSE.md) for details.","ref":"readme.html#license"},{"type":"extras","title":"Overview","doc":"# Hammer, a Rate-Limiter for Elixir\n\nHammer is a rate-limiter for the [Elixir](https://elixir-lang.org/) language.\nIt's killer feature is a pluggable backend system, allowing you to use whichever\nstorage suits your needs. Currently, backends for ETS,\n[Redis](https://github.com/ExHammer/hammer-backend-redis), and [Mnesia](https://github.com/ExHammer/hammer-backend-mnesia) are available.\n\n\n```elixir\n    case Hammer.check_rate(\"file_upload:#{user_id}\", 60_000, 10) do\n      {:allow, _count} ->\n        Upload.file(data)\n      {:deny, _limit} ->\n        render_error_page()\n    end\n```\n\nTo get started with Hammer, read the [Tutorial](/hammer/tutorial.html).\n\nSee the [Hammer.Application module](/hammer/Hammer.Application.html) for full\ndocumentation of configuration options.\n\nA primary goal of the Hammer project is to make it easy to implement new storage\nbackends. See the [documentation on creating\nbackends](/hammer/creatingbackends.html) for more details.","ref":"frontpage.html"},{"type":"extras","title":"New! Hammer-Plug - Overview","doc":"We've just released a new helper-library to make adding rate-limiting to your Phoenix\n(or other plug-based) application even easier: [Hammer.Plug](https://github.com/ExHammer/hammer-plug).","ref":"frontpage.html#new-hammer-plug"},{"type":"extras","title":"Tutorial","doc":"# Tutorial","ref":"tutorial.html"},{"type":"extras","title":"Installation - Tutorial","doc":"Add Hammer as a dependency in `mix.exs`:\n\n```elixir\ndef deps do\n  [{:hammer, \"~> 6.0\"}]\nend\n```","ref":"tutorial.html#installation"},{"type":"extras","title":"Core Concepts - Tutorial","doc":"When we want to rate-limit some action, we want to ensure that the number of\nactions permitted is limited within a specified time-period. For example, a\nmaximum of five times within on minute. Usually the limit is enforced per-user,\nper-client, or per some other unique-ish value, such as IP address. It's much\nrarer, but not unheard-of, to limit the action globally without taking the\nidentity of the user or client into account.\n\nIn the Hammer API, the maximum number of actions is the `limit`, and the\ntimespan (in milliseconds) is the `scale_ms`. The combination of the name of the\naction with some unique identifier is the `id`.\n\nHammer uses a [Token Bucket](https://en.wikipedia.org/wiki/Token_bucket)\nalgorithm to count the number of actions occurring in a \"bucket\". If the count\nwithin the bucket is lower than the limit, then the action is allowed, otherwise\nit is denied.","ref":"tutorial.html#core-concepts"},{"type":"extras","title":"Usage - Tutorial","doc":"To use Hammer, you need to do two things:\n\n- Configure the `:hammer` application\n- Use the functions in the `Hammer` module\n\nIn this example, we will use the `ETS` backend, which stores data in an\nin-memory ETS table.","ref":"tutorial.html#usage"},{"type":"extras","title":"Configuring Hammer - Tutorial","doc":"The Hammer OTP application is configured the usual way, using `Mix.Config`.\nYour project probably has a `config/config.exs` file, in which you should\nconfigure Hammer, like so:\n\n```elixir\nconfig :hammer,\n  backend: {Hammer.Backend.ETS,\n            [expiry_ms: 60_000 * 60 * 4,\n             cleanup_interval_ms: 60_000 * 10]}\n```\n\nThe only configuration key (so far) is `:backend`, and its value is a tuple/pair\nof the backend module name, and a backend-specific keyword list of configuration\noptions.\n\nBecause expiry of stale buckets is so essential to the smooth operation of a\nrate-limiter, all backends will accept an `:expiry_ms` option, and many will\nalso accept `:cleanup_interval_ms`, depending on how expiry is implemented\ninternally.\n\n(For example, Redis supports native data expiry, and so doesn't require\n`:cleanup_interval_ms`.)\n\nThe `:expiry_ms` value should be configured to be longer than the life of the\nlongest bucket you will be using, as otherwise the bucket could be deleted while\nit is still counting up hits for its time period.\n\nThe size of the backend worker pool can be tweaked with the `:pool_size` and\n`:pool_max_overflow` options, (which are then supplied to `poolboy`). `:pool_size`\ndetermines the size of the pool, and `:pool_max_overflow` determines how many extra\nworkers can be spawned when the system is under pressure. The default for both is `0`,\nwhich will be fine for most systems. (Note: we've seen some weird errors sometimes when using a `:pool_max_overflow` higher than zero. Always check how this works for you in production, and consider setting a higher `:pool_size` instead).\n\nLuckily, even if you don't configure `:hammer` at all, the application will\ndefault to the ETS backend anyway, with some sensible defaults.","ref":"tutorial.html#configuring-hammer"},{"type":"extras","title":"The Hammer Module - Tutorial","doc":"Once the Hammer application is running (and it should just start automatically\nwhen your system starts), All you need to do is use the various functions in the\n`Hammer` module:\n\n- `check_rate(id::string, scale_ms::integer, limit::integer)`\n- `check_rate_inc(id::string, scale_ms::integer, limit::integer, increment::integer)`\n- `inspect_bucket(id::string, scale_ms::integer, limit::integer)`\n- `delete_buckets(id::string)`\n- `make_rate_checker(id_prefix, scale_ms, limit)`\n\nThe most interesting is `check_rate`, which checks if the rate-limit for the\ngiven `id` has been exceeded in the specified time-`scale`.\n\nIdeally, the `id` should be a combination of some action-specific, descriptive\nprefix with some data which uniquely identifies the user or client performing\nthe action.\n\nExample:\n\n```elixir\n# limit file uploads to 10 per minute per user\nuser_id = get_user_id_somehow()\ncase Hammer.check_rate(\"upload_file:#{user_id}\", 60_000, 10) do\n  {:allow, _count} ->\n    # upload the file\n  {:deny, _limit} ->\n    # deny the request\nend\n```","ref":"tutorial.html#the-hammer-module"},{"type":"extras","title":"Custom increments - Tutorial","doc":"The `Hammer` module also includes  a `check_rate_inc` function, which allows you\nto specify the number by which to increment the current bucket. This is useful\nfor rate-limiting APIs which have some idea of \"cost\", where the cost of a given\noperation can be determined and expressed as an integer.\n\nExample:\n\n```elixir\n# Bulk file upload\nuser_id = get_user_id_somehow()\nn = get_number_of_files()\ncase Hammer.check_rate_inc(\"upload_file_bulk:#{user_id}\", 60_000, 10, n) do\n  {:allow, _count} ->\n    # upload all of the files\n  {:deny, _limit} ->\n    # deny the request\nend\n```","ref":"tutorial.html#custom-increments"},{"type":"extras","title":"Switching to Redis - Tutorial","doc":"There may come a time when ETS just doesn't cut it, for example if we end up\nload-balancing across many nodes and want to keep our rate-limiter state in one\ncentral store. [Redis](https://redis.io) is ideal for this use-case, and\nfortunately Hammer supports\na [Redis backend](https://github.com/ExHammer/hammer-backend-redis).\n\nTo change our application to use the Redis backend, we only need to install the\nredis backend package, and change the `:backend` tuple that is used to configure\nthe `:hammer` application:\n\n```elixir\n# config :hammer,\n#   backend: {Hammer.Backend.ETS, []}\n\nconfig :hammer,\n  backend: {Hammer.Backend.Redis, [expiry_ms: 60_000 * 60 * 2,\n                                   redix_config: [host: \"localhost\",\n                                                  port: 6379],\n                                   pool_size: 4,\n                                   pool_max_overflow: 2]}\n```\n\nThen it should all Just Work™.\n\n\n## (Advanced) using multiple backends at the same time\n\nHammer can be configured to start multiple backends, which can then be referred\nto separately when checking a rate-limit. In this example we configure both and\nETS backend under the key `:in_memory`, and a Redis backend under the key\n`:redis`...\n\n```elixir\n\n\nconfig :hammer,\n  backend: [\n    in_memory: {Hammer.Backend.ETS, [expiry_ms: 60_000 * 60 * 2]},\n    redis: {Hammer.Backend.Redis, [expiry_ms: 60_000 * 60 * 2,\n                                    redix_config: [host: \"localhost\",\n                                                    port: 6379]]}\n  ]\n```\n\nWe can then refer to these backends separately:\n\n```elixir\nHammer.check_rate(:in_memory, \"upload:#{user_id}\", 60_000, 5)\nHammer.check_rate(:redis,     \"upload:#{user_id}\", 60_000, 5)\n```\n\nWhen using multiple backends the backend specifier key is mandatory, there is no\nnotion of a default backend.\n\nIn version 4.0 and up, it is even possible to have multiple instances of the same\nbackend type, like so:\n\n```elixir\nconfig :hammer,\n  backend: [\n    redis_one: {Hammer.Backend.Redis, [expiry_ms: 60_000 * 60 * 2,\n                                       redix_config: [host: \"localhost\",\n                                                      port: 6666]]}\n    redis_two: {Hammer.Backend.Redis, [expiry_ms: 60_000 * 60 * 5,\n                                       redix_config: [host: \"localhost\",\n                                                      port: 7777]]}\n  ]\n```","ref":"tutorial.html#switching-to-redis"},{"type":"extras","title":"Further Reading - Tutorial","doc":"See the docs for the [Hammer](/hammer/Hammer.html) module for full documentation\non all the functions created by `use Hammer`.\n\nSee the [Hammer.Application](/hammer/Hammer.Application.html) for all\nconfiguration options.\n\nAlso, consult the documentation for the backend you are using, for any extra\nconfiguration options that may be relevant.\n\nSee the [Creating Backends](/hammer/creatingbackends.html) for information on\ncreating new backends for Hammer.","ref":"tutorial.html#further-reading"},{"type":"extras","title":"Creating Backends","doc":"# Creating Backends\n\n\nSee `Hammer.Backend.ETS` for a realistic example of a Hammer Backend module.\n\nBackends for Hammer are expected to implement the\n[Hammer.Backend](/hammer/Hammer.Backend.html) behaviour, and be named something\nlike `Hammer.Backend.Foo`. Several instances of the backend worker will be\nstarted as a worker pool (using the poolboy library), and so backend processes\nshould not assume that there is only one instance of this server process.","ref":"creatingbackends.html"},{"type":"extras","title":"The `Hammer.Backend` behaviour - Creating Backends","doc":"The expected backend api is as follows:\n\n#","ref":"creatingbackends.html#the-hammer-backend-behaviour"},{"type":"extras","title":"start_link(args) - Creating Backends","doc":"- `args`: Keyword list of configuration.\n\nThe `expiry_ms` key is considered essential, as the backend process should\ndelete expired buckets somehow. Depending on how the backend implements cleanup,\nthe `cleanup_interval_ms` key may be required. Other keys are for the backend\ndeveloper to choose, but for example, if the backend were using a database\ncalled `FooDB` to store buckets, then an appropriate key would be\n`foodb_config`.\n\nExample:\n\n```elixir\n  Hammer.Backend.Foo.start_link(expiry_ms: 60_000 * 60,\n                                cleanup_interval_ms: 60_000 * 10,\n                                foodb_config: [host: \"localhost\"])\n```\n\n#","ref":"creatingbackends.html#start_link-args"},{"type":"extras","title":"count_hit(pid, key, timestamp) - Creating Backends","doc":"- `pid`: pid of the backend process being called\n- `key`: The key of the current bucket, in the form of a tuple `{bucket::integer, id::String}`.\n- `timestamp`: The current timestamp (integer)\n\nThis function should increment the count in the bucket by 1.\n\nReturns: Either a Tuple of `{:ok, count}` where count is the current count of the bucket,\nor `{:error, reason}`.\n\n\n#","ref":"creatingbackends.html#count_hit-pid-key-timestamp"},{"type":"extras","title":"count_hit(pid, key, timestamp, increment) - Creating Backends","doc":"- `pid`: pid of the backend process being called\n- `key`: The key of the current bucket, in the form of a tuple `{bucket::integer, id::String}`.\n- `timestamp`: The current timestamp (integer)\n- `increment`: The number by which to increment the bucket\n\nThis function should increment the count in the bucket by the specified increment.\n\nReturns: Either a Tuple of `{:ok, count}` where count is the current count of the bucket,\nor `{:error, reason}`.\n\n\n#","ref":"creatingbackends.html#count_hit-pid-key-timestamp-increment"},{"type":"extras","title":"get_bucket(pid, key) - Creating Backends","doc":"- `pid`: pid of the backend process being called\n- `key`: The key of the current bucket, in the form of a tuple `{bucket::integer, id::String}`.\n\nReturns: Either a tuple of `{:ok, bucket}`, where `bucket` is a tuple of\n`{key, count, created_at, updated_at}`, key is, as usual, a tuple of `{bucket_number, id}`,\n`count` is the count of hits in the bucket, `created_at` and `updated_at` are integer timestamps,\nor `{:error, reason}`\n\n\n#","ref":"creatingbackends.html#get_bucket-pid-key"},{"type":"extras","title":"delete_buckets(pid, id) - Creating Backends","doc":"- `pid`: pid of the backend process being called\n- `id`: rate-limit id (string) to delete\n\nThis should delete all existing buckets associated with the supplied `id`.\n\nReturns: Either `{:ok, count}`, or `{:error, reason}`","ref":"creatingbackends.html#delete_buckets-pid-id"}],"content_type":"text/markdown"}